package tcc.haruki.domain.xtr;

import java.math.BigInteger;

import tcc.haruki.domain.XTRElem;
import tcc.haruki.domain.XTRTrincaElem;
import tcc.haruki.exceptions.XTRInvalidParametersException;
import tcc.haruki.exceptions.XTRInvalidKeyException;

public class XTRPublicKey {
	private XTRParameters params;
	private XTRElem trg;
	private XTRTrincaElem h;
	
	/**
	 * @param params parâmetros XTR
	 * @param trg Tr(g)
	 * @param h Tr(g<sup>k</sup>), onde k faz parte da chave privada
	 * @throws XTRInvalidKeyException 
	 * @throws XTRInvalidParametersException 
	 */
	public XTRPublicKey(XTRParameters params, XTRElem trg, XTRTrincaElem h) throws XTRInvalidKeyException, XTRInvalidParametersException {
		this.params = params;
		this.trg = trg;
		this.h = h;
		
		if (this.params == null || this.trg == null || this.h == null) {
			throw new XTRInvalidKeyException("Algum valor da chave é nulo");
		}
		
		if (!this.params.isValid()) {
			throw new XTRInvalidParametersException();
		}
	}

	/**
	 * p dos parâmetros XTR.
	 * @return p
	 */
	public BigInteger getP() {
		if (this.params == null) {
			return null;
		}
		return this.params.getP();
	}
	
	/**
	 * q dos parâmetros XTR.
	 * @return q
	 */
	public BigInteger getQ() {
		if (this.params == null) {
			return null;
		}
		return this.params.getQ();
	}
	
	/**
	 * 
	 * @return Tr(g)
	 */
	public XTRElem getTrg() {
		return trg;
	}

	/**
	 * 
	 * @return h = (Tr(g<sup>k-1</sup>), Tr(g<sup>k</sup>), Tr(g<sup>k+1</sup>))
	 */
	public XTRTrincaElem getH() {
		return h;
	}
	
	
}
