package tcc.haruki.domain;

import java.math.BigInteger;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Test;
import org.junit.rules.*;
import org.junit.experimental.categories.Categories.ExcludeCategory;

public class GaloisFieldTest {
	private GaloisField gf;
	private BigInteger p;
	
	@Before
	public void setup() {
		p = new BigInteger("179");
		this.gf = new GaloisField(p);
	}
	
	@Test
	public void valTest() {
		BigInteger num1 = new BigInteger("150");
		BigInteger num2 = num1.add(p);

		/* Testando se o número é ele mesmo, caso 0 <= n < p */
		Assert.assertEquals(BigInteger.ZERO, gf.val(BigInteger.ZERO));
		Assert.assertEquals(num1, gf.val(num1));
		
		/* Testando se p == 0 */
		Assert.assertEquals(BigInteger.ZERO, gf.val(p));
		
		/* Testando se "dá a volta", ou seja, para 0 <= x < p: x+p == x */
		Assert.assertEquals(num1, gf.val(num2));
	}
	
	@Test
	public void inverseTest() {
		BigInteger num1 = new BigInteger("150");
		BigInteger num2 = new BigInteger("37");
		
		Assert.assertEquals(num2, gf.inverse(num1));
		
		Assert.assertEquals(BigInteger.ONE, gf.inverse(BigInteger.ONE));
	}
	
	@Test(expected=ArithmeticException.class)
	public void inverseExceptionOnInversePTest() throws Exception {
		Assert.assertNull(gf.inverse(p));
	}
	
	@Test(expected=ArithmeticException.class)
	public void inverseExceptionOnInverseZeroTest() throws Exception {
		Assert.assertNull(gf.inverse(BigInteger.ZERO));
	}
	
	@Test
	public void addTest() {
		BigInteger num1 = new BigInteger("150");
		BigInteger num2 = new BigInteger("151");
		
		Assert.assertEquals(num1, gf.add(num1, BigInteger.ZERO));
		
		Assert.assertEquals(num2, gf.add(num1, BigInteger.ONE));
		
		Assert.assertEquals(num1, gf.add(num1, p));
	}
	
	@Test
	public void mulTest() {
		BigInteger num1 = new BigInteger("150");
		BigInteger num2 = new BigInteger("75");
		
		BigInteger num3 = new BigInteger("121");
		
		Assert.assertEquals(BigInteger.ZERO, gf.mul(num1, BigInteger.ZERO));
		Assert.assertEquals(num1, gf.mul(num1, BigInteger.ONE));
		
		Assert.assertEquals(num1, gf.mul(num2, new BigInteger("2")));
		
		Assert.assertEquals(num3, gf.mul(num1, new BigInteger("2")));
	}
	
	@Test
	public void scalarExpTest() {
		BigInteger num1 = new BigInteger("3");
		BigInteger num2 = new BigInteger("2");
		BigInteger num3 = new BigInteger("9");
		BigInteger num4 = new BigInteger("27");
		
		Assert.assertEquals(BigInteger.ONE, gf.scalarExp(num1, BigInteger.ZERO));
		Assert.assertEquals(num3, gf.scalarExp(num1, num2));
		Assert.assertEquals(num4, gf.scalarExp(num1, num1));
	}
}
